@@ -1,5 +1,24 @@
 # Created on 26Sep10 by Dan Putler
-# Last modified on 26Feb11 by Dan Putler
+# Last modified on 10Nov11 by Dan Putler
+
+# .onAttach function to place .subset, .targetLevel, and .trueResp into the
+# Rcmdr environment. Hopefully this works. This is done to deal with R2.14.0
+.onAttach <- function(libname, pkgname){
+    if (!interactive()) return()
+    Rcmdr <- options()$Rcmdr
+    plugins <- Rcmdr$plugins
+    if ((!pkgname %in% plugins) && !getRcmdr("autoRestart")) {
+        Rcmdr$plugins <- c(plugins, pkgname)
+        options(Rcmdr=Rcmdr)
+        closeCommander(ask=FALSE, ask.save=TRUE)
+        putRcmdr(".subset", gettextRcmdr("<all valid cases>"))
+        putRcmdr(".Sample", NULL)
+        putRcmdr(".targetLevel", NULL)
+        putRcmdr(".trueResp", NULL)
+        Commander()
+        }
+    palette(c("#0072B2","#D55E00","#000000","#F0E442","#2B9F78","#E69F00","#5684E9","#CC79A7"))
+    }
 
 # .First.lib function to place .subset, .targetLevel, and .trueResp into the
 # Rcmdr environment. Hopefully this works.
@@ -486,6 +505,260 @@ SDIndexPlot <- function(){
     dialogSuffix(rows=3, columns=3)
     }
 
+# K-Centroids Clustering
+kcentroidsClustering <- function(){
+    initializeDialog(title=gettextRcmdr("K-Centroids Clustering"))
+    dataFrame <- tkframe(top)
+    xBox <- variableListBox(dataFrame, Numeric(), selectmode="multiple",
+      title=gettextRcmdr("Variables (pick one or more)"))
+    subsetBoxBCA(dataFrame)
+    assignFrame <- tkframe(dataFrame)
+    assignClusters <- tclVar("0")
+    assignCB <- tkcheckbutton(assignFrame)
+    tkconfigure(assignCB, variable=assignClusters)
+    assignName <- tclVar("KCentroids")
+    assignField <- tkentry(assignFrame, width="15",
+      textvariable=assignName)
+    radioButtons(name="method", buttons=c("kmn", "kmd", "neuralgas"),
+      labels=gettextRcmdr(c("K-Means", "K-Medians", "Neural Gas")),
+      title=gettextRcmdr("Clustering Method"))
+    optionsFrame <- tkframe(top)
+    clusterNumber <- tclVar("2")
+    clusterNumSlider <- tkscale(optionsFrame, from=2, to=10, showvalue=TRUE,
+      variable=clusterNumber, resolution=1, orient="horizontal")
+    seedNumber <- tclVar("10")
+    seedNumSlider <- tkscale(optionsFrame, from=1, to=20, showvalue=TRUE,
+      variable=seedNumber, resolution=1, orient="horizontal")
+    #iterNumber <- tclVar("10")
+    #iterNumSlider <- tkscale(optionsFrame, from=5, to=30, showvalue=TRUE,
+    #  variable=iterNumber, resolution=5, orient="horizontal")
+    summaryClusters <- tclVar("1")
+    summaryCB <- tkcheckbutton(optionsFrame)
+    tkconfigure(summaryCB, variable=summaryClusters)
+    plotCls2d <- tclVar("1")
+    plot2dCB <- tkcheckbutton(optionsFrame)
+    tkconfigure(plot2dCB, variable=plotCls2d)
+    plotCls3d <- tclVar("0")
+    plot3dCB <- tkcheckbutton(optionsFrame)
+    tkconfigure(plot3dCB, variable=plotCls3d)
+    plotFrame <- tkframe(optionsFrame)
+    plotPts <- tclVar("1")
+    plotPtsCB <- tkcheckbutton(plotFrame)
+    tkconfigure(plotPtsCB, variable=plotPts)
+    plotCnt <- tclVar("0")
+    plotCntCB <- tkcheckbutton(plotFrame)
+    tkconfigure(plotCntCB, variable=plotCnt)
+    onOK <- function(){
+        x <- getSelection(xBox)
+        nvar <- length(x)
+        subset <- trim.blanks(tclvalue(subsetVariable))
+        clusMethod <- tclvalue(methodVariable)
+        nClusters <- tclvalue(clusterNumber)
+        seeds <- tclvalue(seedNumber)
+        #iters <- tclvalue(iterNumber)
+        clusterSummary <- tclvalue(summaryClusters)
+        clsPlot2d <- tclvalue(plotCls2d)
+        clsPlot3d <- tclvalue(plotCls3d)
+        ptsPlot <- tclvalue(plotPts)
+        cntPlot <- tclvalue(plotCnt)
+        clusterAssign <- tclvalue(assignClusters)
+        clusterVariable <- trim.blanks(tclvalue(assignName))
+        if (clusterAssign == "1" & !is.valid.name(clusterVariable)){
+            errorCondition(recall=kcentroidsClustering, message=
+              sprintf(gettextRcmdr('"%s" is not a valid name.'),
+	      clusterVariable))
+	      return()
+            }
+        closeDialog()
+        if (clusterAssign == "1"){
+           if (is.element(clusterVariable, Variables())) {
+                if ("no" == tclvalue(checkReplace(clusterVariable))){
+                    kcentroidsClustering()
+                    return()
+                    }
+                }
+           } 
+        if (length(x)==0) {
+            errorCondition(recall=kcentroidsClustering, 
+              message==gettextRcmdr("No variables selected."))
+            return()
+            }
+        varFormula <- paste(x, collapse=" + ")
+        vars <- paste(x, collapse=",", sep="")
+        .activeDataSet <- ActiveDataSet()
+        dset <- if (trim.blanks(subset) == gettextRcmdr("<all valid cases>")) .activeDataSet
+          else {paste(.activeDataSet, "[", .activeDataSet, "$", subset, ", ]",
+            sep="")}
+        xmat <- paste("model.matrix(~-1 + ", varFormula, ", ", dset, ")",
+          sep="")
+        details <- 'FUN = kcca, family = kccaFamily("kmeans")'
+        if(clusMethod == "kmd") {
+            details <- 'FUN = kcca, family = kccaFamily("kmedians")'
+            }
+        else if(clusMethod == "neuralgas") {
+            details <- 'FUN = cclust, dist = "euclidean", method = "neuralgas"'
+            }
+        command <- paste("stepFlexclust(", xmat, ", k = ", nClusters,
+          ", nrep = ", seeds, ", ", details, ")", sep="")
+        assign(".cluster", justDoIt(command), envir=.GlobalEnv)
+        logger(paste(".cluster <- ", command, sep=""))
+        if (clusterSummary == "1") {
+            doItAndPrint("summary(.cluster)")
+            doItAndPrint("print(.cluster@centers) # Cluster Centroids")
+            }
+	if((clsPlot2d=="1" | clsPlot3d=="1") & ptsPlot=="0" & cntPlot=="0") {
+	   Message(gettextRcmdr("Bi-plot(s) contains no points or centroids."),
+	     type="warning")
+	   }
+	ptsOpt <- "FALSE"
+	if(ptsPlot == "1") ptsOpt <- "TRUE"
+	cntOpt <- "FALSE"
+	if(cntPlot == "1") cntOpt <- "TRUE"
+        if (clsPlot2d == "1") {
+            plotCmd2d <- paste("bpCent(prcomp(", xmat, 
+              "), clusters(.cluster), data.pts = ", ptsOpt, ", centroids = ",
+	          cntOpt, ", xlabs = as.character(clusters(.cluster)))", sep="")
+           justDoIt(plotCmd2d)
+           logger(plotCmd2d)
+           }
+        if (clsPlot3d == "1") {
+            plotCmd3d <- paste("bpCent3d(prcomp(", xmat, 
+              "), clusters(.cluster), data.pts = ", ptsOpt, ", centroids = ",
+	          cntOpt, ", xlabs = as.character(clusters(.cluster)))", sep="")
+           justDoIt(plotCmd3d)
+           logger(plotCmd3d)
+           .Tcl("update")
+           activateMenus()
+           rgl.bringtotop()
+           }
+        if (clusterAssign == "1") {
+#            assignCommand <- paste(.activeDataSet, "$", clusterVariable,
+#              " <- assignCluster(", xmat, ", ", .activeDataSet,
+#              ", clusters(.cluster))", sep="")
+            assignCommand <- paste(.activeDataSet, "$", clusterVariable,
+              " <- as.factor(clusters(.cluster))", sep="")
+            justDoIt(assignCommand)
+            logger(assignCommand)
+            activeDataSet(.activeDataSet)
+            }
+        justDoIt(paste("remove(.cluster)"))
+        logger(paste("remove(.cluster)"))
+        activateMenus()
+        tkfocus(CommanderWindow())
+        }
+    OKCancelHelp(helpSubject="kcca")
+    tkgrid(getFrame(xBox), sticky="nw")
+    tkgrid(subsetFrame, sticky="w")
+    tkgrid(tklabel(assignFrame, 
+      text=gettextRcmdr("Assign clusters to\nthe data set         ")),
+      assignCB, sticky="w")
+    tkgrid(tklabel(assignFrame, text=gettextRcmdr("Assignment variable: ")),
+      assignField, sticky="w")
+    tkgrid(assignFrame, sticky="w")
+    tkgrid(tklabel(optionsFrame, text=gettextRcmdr("Number of clusters:")),
+      clusterNumSlider, sticky="sw")
+    tkgrid(tklabel(optionsFrame, text=gettextRcmdr("Number of starting seeds:")),
+      seedNumSlider, sticky="sw")
+    #tkgrid(tklabel(optionsFrame, text=gettextRcmdr("Maximum iterations:")),
+    #  iterNumSlider, sticky="sw")
+    tkgrid(tklabel(optionsFrame, 
+      text=gettextRcmdr("Print cluster summary")), summaryCB, sticky="w")
+    tkgrid(tklabel(optionsFrame, 
+      text=gettextRcmdr("2D bi-plot of clusters")), plot2dCB, sticky="w")
+    tkgrid(tklabel(optionsFrame, 
+      text=gettextRcmdr("3D bi-plot of clusters")), plot3dCB, sticky="w")
+    tkgrid(tklabel(plotFrame, text=gettextRcmdr("Plot points")), plotPtsCB,
+      tklabel(plotFrame, text=gettextRcmdr("Plot centroids")), plotCntCB,
+      sticky="w")
+    tkgrid(plotFrame, columnspan=2, sticky="w")
+    tkgrid(dataFrame, tklabel(top, text="  "), methodFrame, optionsFrame,
+        sticky="nw")
+    tkgrid(buttonsFrame, columnspan=3, sticky="w")
+    dialogSuffix(rows=3, columns=4)
+    }
+
+# The bootstrap replicate diagnostic testing for K-Centroids
+bootDiagnostics <- function(){
+    initializeDialog(title=gettextRcmdr("K-Centroids Clustering Diagnostics"))
+    dataFrame <- tkframe(top)
+    xBox <- variableListBox(dataFrame, Numeric(), selectmode="multiple",
+      title=gettextRcmdr("Variables (pick one or more)"))
+    subsetBoxBCA(dataFrame)
+    radioButtons(name="method", buttons=c("kmn", "kmd", "neuralgas"),
+      labels=gettextRcmdr(c("K-Means", "K-Medians", "Neural Gas")),
+      title=gettextRcmdr("Clustering Method"))
+    optionsFrame <- tkframe(top)
+    minCluster <- tclVar("2")
+    minClusterSlider <- tkscale(optionsFrame, from=2, to=8, showvalue=TRUE,
+      variable=minCluster, resolution=1, orient="horizontal")
+    maxCluster <- tclVar("4")
+    maxClusterSlider <- tkscale(optionsFrame, from=3, to=10, showvalue=TRUE,
+      variable=maxCluster, resolution=1, orient="horizontal")
+    bootRep <- tclVar("100")
+    bootRepSlider <- tkscale(optionsFrame, from=50, to=200, showvalue=TRUE,
+      variable=bootRep, resolution=10, orient="horizontal")
+    seedNumber <- tclVar("3")
+    seedNumSlider <- tkscale(optionsFrame, from=1, to=5, showvalue=TRUE,
+      variable=seedNumber, resolution=1, orient="horizontal")
+    .activeDataset <- ActiveDataSet()
+     onOK <- function(){
+        x <- getSelection(xBox)
+        nvar <- length(x)
+        subset <- tclvalue(subsetVariable)
+        clusMethod <- tclvalue(methodVariable)
+        minClus <- tclvalue(minCluster)
+        maxClus <- tclvalue(maxCluster)
+        bReps <- tclvalue(bootRep)
+        seeds <- tclvalue(seedNumber)
+        closeDialog()
+        if (length(x)==0) {
+            errorCondition(recall=bootDiagnostics, 
+              message=gettextRcmdr("No variables selected."))
+            return()
+            }
+        if (as.numeric(minClus) > as.numeric(maxClus)) {
+            errorCondition(recall=SDIndexPlot, message=gettextRcmdr(
+              "The minimum number of clusters selected exceeds the maximum."))
+            return()
+            }
+        varFormula <- paste(x, collapse=" + ")
+        vars <- paste(x, collapse=",", sep="")
+        dset <- if (subset == gettextRcmdr("<all valid cases>")) .activeDataSet
+          else {paste(.activeDataSet, "[", .activeDataSet, "$", subset, ", ]",
+            sep="")}
+        xmat <- paste("model.matrix(~-1 + ", varFormula, ", ", dset, ")",
+          sep="")
+        details <- 'FUN = cclust, dist = "euclidean", method = "kmeans"'
+        if(clusMethod == "kmd") {
+            details <- 'FUN = kcca, family = kccaFamily("kmedians")'
+            }
+        else if(clusMethod == "neuralgas") {
+            details <- 'FUN = cclust, dist = "euclidean", method = "neuralgas"'
+            }
+        command <- paste("bootCVD(", xmat, ", k = ", minClus, ":", maxClus,
+            ", nboot = ", bReps, ", nrep = ", seeds, ', method = "', clusMethod,
+            '", col1 = 1, col2 = 2, dsname = "', .activeDataSet, '")', sep="")
+        doItAndPrint(command)
+        activateMenus()
+        tkfocus(CommanderWindow())
+        }
+    OKCancelHelp(helpSubject="bootCVD")
+    tkgrid(getFrame(xBox), sticky="nw")
+    tkgrid(subsetFrame, sticky="w")
+    tkgrid(tklabel(optionsFrame, text=gettextRcmdr("Minimum number of clusters:")),
+      minClusterSlider, sticky="sw")
+    tkgrid(tklabel(optionsFrame, text=gettextRcmdr("Maximum number of clusters:")),
+      maxClusterSlider, sticky="sw")
+    tkgrid(tklabel(optionsFrame, text=gettextRcmdr("Bootstrap replicates:")),
+      bootRepSlider, sticky="sw")
+    tkgrid(tklabel(optionsFrame, text=gettextRcmdr("Number of random seeds:")),
+      seedNumSlider, sticky="sw")
+    tkgrid(dataFrame, tklabel(top, text="  "), methodFrame, optionsFrame, sticky="nw")
+    tkgrid(buttonsFrame, columnspan=3, sticky="w")
+    dialogSuffix(rows=3, columns=4)
+    }
+
+# List hierachical clustering solutions
 listHclustSolutions <- function(envir=.GlobalEnv, ...) {
     objects <- ls(envir=envir, ...)
     if (length(objects) == 0) NULL
@@ -638,6 +911,7 @@ hclustSummaryBCA <- function(){
     plotPtsCB <- tkcheckbutton(plotFrame)
     tkconfigure(plotPtsCB, variable=plotPts)
     plotCnt <- tclVar("0")
+
     plotCntCB <- tkcheckbutton(plotFrame)
     tkconfigure(plotCntCB, variable=plotCnt)
     if(length(hclustObjects)==0) {
@@ -1005,6 +1279,7 @@ nnetModel <- function(){
     modelFormula()
     subsetBox(model=TRUE)
     tkgrid(getFrame(xBox), sticky="w")
+
     tkgrid(formulaFrame, sticky="w")
     tkgrid(subsetFrame, sticky="w")
     tkgrid(tklabel(optionsFrame, text=gettextRcmdr("Hidden layer units:")),
@@ -1746,207 +2021,3 @@ helpAboutBCA <- function() {
 	else help("RcmdrPlugin.BCA")
 }
 
-
-## The scatter plot dialog boxes
-scatterPlotBCA <- function(){
-    require("BCA")
-    initializeDialog(title=gettextRcmdr("Scatterplot"))
-    .numeric <- Numeric()
-    variablesFrame <- tkframe(top)
-    xBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("x-variable (pick one)"))
-    yBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("y-variable (pick one)"))
-    optionsParFrame <- tkframe(top)
-    checkBoxes(window=optionsParFrame, frame="optionsFrame", 
-		boxes=c("identify", "jitterX", "jitterY", "logX", "logY", "boxplots", "lsLine", "smoothLine", "spread"),
-        initialValues=c(0, 0, 0, 0, 0, 1, 1, 1, 0), 
-		labels=gettextRcmdr(c("Identify points", "Jitter x-variable", "Jitter y-variable", "Log x-axis", "Log y-axis",
-        "Marginal boxplots", "Least-squares line", "Smooth line", "Show spread")), title="Options")
-    sliderValue <- tclVar("50")
-    slider <- tkscale(optionsFrame, from=0, to=100, showvalue=TRUE, variable=sliderValue,
-        resolution=5, orient="horizontal")
-    subsetBox()
-    labelsFrame <- tkframe(top)
-    xlabVar <- tclVar(gettextRcmdr("<auto>"))
-    ylabVar <- tclVar(gettextRcmdr("<auto>"))
-    xlabFrame <- tkframe(labelsFrame)
-    xlabEntry <- ttkentry(xlabFrame, width="25", textvariable=xlabVar)
-    xlabScroll <- ttkscrollbar(xlabFrame, orient="horizontal",
-        command=function(...) tkxview(xlabEntry, ...))
-    tkconfigure(xlabEntry, xscrollcommand=function(...) tkset(xlabScroll, ...))
-    tkgrid(labelRcmdr(xlabFrame, text=gettextRcmdr("x-axis label"), fg="blue"), sticky="w")
-    tkgrid(xlabEntry, sticky="w")
-    tkgrid(xlabScroll, sticky="ew")
-    ylabFrame <- tkframe(labelsFrame)
-    ylabEntry <- ttkentry(ylabFrame, width="25", textvariable=ylabVar)
-    ylabScroll <- ttkscrollbar(ylabFrame, orient="horizontal",
-        command=function(...) tkxview(ylabEntry, ...))
-    tkconfigure(ylabEntry, xscrollcommand=function(...) tkset(ylabScroll, ...))
-    tkgrid(labelRcmdr(ylabFrame, text=gettextRcmdr("y-axis label"), fg="blue"), sticky="w")
-    tkgrid(ylabEntry, sticky="w")
-    tkgrid(ylabScroll, sticky="ew")
-    tkgrid(xlabFrame, labelRcmdr(labelsFrame, text="     "), ylabFrame, sticky="w")
-    parFrame <- tkframe(optionsParFrame)
-    pchVar <- tclVar(gettextRcmdr("<auto>"))
-    pchEntry <- ttkentry(parFrame, width=25, textvariable=pchVar)
-    cexValue <- tclVar("1")
-    cex.axisValue <- tclVar("1")
-    cex.labValue <- tclVar("1")
-    cexSlider <- tkscale(parFrame, from=0.5, to=2.5, showvalue=TRUE, variable=cexValue,
-        resolution=0.1, orient="horizontal")
-    cex.axisSlider <- tkscale(parFrame, from=0.5, to=2.5, showvalue=TRUE, variable=cex.axisValue,
-        resolution=0.1, orient="horizontal")
-    cex.labSlider <- tkscale(parFrame, from=0.5, to=2.5, showvalue=TRUE, variable=cex.labValue,
-        resolution=0.1, orient="horizontal")
-    onOK <- function(){
-        x <- getSelection(xBox)
-        y <- getSelection(yBox)
-        closeDialog()
-        if (length(x) == 0 || length(y) == 0){
-            errorCondition(recall=scatterPlot, message=gettextRcmdr("You must select two variables"))
-            return()
-            }
-        if (x == y) {
-            errorCondition(recall=scatterPlot, message=gettextRcmdr("x and y variables must be different"))
-            return()
-            }
-        .activeDataSet <- ActiveDataSet()
-        jitter <- if ("1" == tclvalue(jitterXVariable) && "1" == tclvalue(jitterYVariable)) ", jitter=list(x=1, y=1)"
-            else if ("1" == tclvalue(jitterXVariable)) ", jitter=list(x=1)"
-            else if ("1" == tclvalue(jitterYVariable)) ", jitter=list(y=1)"
-            else ""
-		logstring <- ""
-		if ("1" == tclvalue(logXVariable)) logstring <- paste(logstring, "x", sep="")
-		if ("1" == tclvalue(logYVariable)) logstring <- paste(logstring, "y", sep="")
-		log <- if(logstring != "") paste(', log="', logstring, '"', sep="") else ""
-		if("1" == tclvalue(identifyVariable)){
-			RcmdrTkmessageBox(title="Identify Points",
-					message=paste(gettextRcmdr("Use left mouse button to identify points,\n"),
-						gettextRcmdr(if (MacOSXP()) "esc key to exit." else "right button to exit."), sep=""),
-					icon="info", type="ok")
-			idtext <- ', id.method="identify"'
-		}
-        else idtext <- ""
-        box <- if ("1" == tclvalue(boxplotsVariable)) "'xy'" else "FALSE"
-        line <- if("1" == tclvalue(lsLineVariable)) "lm" else "FALSE"
-        smooth <- as.character("1" == tclvalue(smoothLineVariable))
-		spread <- as.character("1" == tclvalue(spreadVariable))
-        span <- as.numeric(tclvalue(sliderValue))
-        subset <- tclvalue(subsetVariable)
-        subset <- if (trim.blanks(subset) == gettextRcmdr("<all valid cases>")) ""
-            else paste(", subset=", subset, sep="")
-        xlab <- trim.blanks(tclvalue(xlabVar))
-        xlab <- if(xlab == gettextRcmdr("<auto>")) "" else paste(', xlab="', xlab, '"', sep="")
-        ylab <- trim.blanks(tclvalue(ylabVar))
-        ylab <- if(ylab == gettextRcmdr("<auto>")) "" else paste(', ylab="', ylab, '"', sep="")
-        cex <- as.numeric(tclvalue(cexValue))
-        cex <- if(cex == 1) "" else paste(', cex=', cex, sep="")
-        cex.axis <- as.numeric(tclvalue(cex.axisValue))
-        cex.axis <- if(cex.axis == 1) "" else paste(', cex.axis=', cex.axis, sep="")
-        cex.lab <- as.numeric(tclvalue(cex.labValue))
-        cex.lab <- if(cex.lab == 1) "" else paste(', cex.lab=', cex.lab, sep="")
-        pch <- gsub(" ", ",", tclvalue(pchVar))
-        if ("" == pch) {
-            errorCondition(recall=scatterPlot, message=gettextRcmdr("No plotting characters."))
-            return()
-            }
-        pch <- if(trim.blanks(pch) == gettextRcmdr("<auto>")) "" else paste(", pch=c(", pch, ")", sep="")
-        if (.groups == FALSE) {
-            doItAndPrint(paste("scatterplotBCA(", y, "~", x, log,
-                ", reg.line=", line, ", smooth=", smooth, ", spread=", spread, idtext,
-                ", boxplots=", box, ", span=", span/100, jitter, xlab, ylab,
-                cex, cex.axis, cex.lab, pch,
-                ", data=", .activeDataSet, subset, ")", sep=""))
-            }
-        else {
-            doItAndPrint(paste("scatterplotBCA(", y, "~", x," | ", .groups,
-                ", reg.line=", line, ", smooth=", smooth, ", spread=", spread, idtext,
-                ", boxplots=", box, ", span=", span/100, jitter, xlab, ylab,
-                cex, cex.axis, cex.lab, pch,
-                ", by.groups=", .linesByGroup,
-                ", data=", .activeDataSet, subset, ")", sep=""))
-            }
-        activateMenus()
-        tkfocus(CommanderWindow())
-        }
-    groupsBox(scatterPlot, plotLinesByGroup=TRUE)
-    OKCancelHelp(helpSubject="scatterplot")
-    tkgrid(getFrame(xBox), getFrame(yBox), sticky="nw")
-    tkgrid(variablesFrame, sticky="w")
-    tkgrid(labelRcmdr(optionsFrame, text=gettextRcmdr("Span for smooth")), slider, sticky="w")
-    tkgrid(labelRcmdr(parFrame, text=gettextRcmdr("Plotting Parameters"), fg="blue"), sticky="w")
-    tkgrid(labelRcmdr(parFrame, text=gettextRcmdr("Plotting characters")), pchEntry, stick="w")
-    tkgrid(labelRcmdr(parFrame, text=gettextRcmdr("Point size")), cexSlider, sticky="w")
-    tkgrid(labelRcmdr(parFrame, text=gettextRcmdr("Axis text size")), cex.axisSlider, sticky="w")
-    tkgrid(labelRcmdr(parFrame, text=gettextRcmdr("Axis-labels text size")), cex.labSlider, sticky="w")
-    tkgrid(optionsFrame, parFrame, sticky="nw")
-    tkgrid(optionsParFrame, sticky="w")
-    tkgrid(labelsFrame, sticky="w")
-    tkgrid(subsetFrame, sticky="w")
-    tkgrid(groupsFrame, sticky="w")
-    tkgrid(labelRcmdr(top, text=" "))
-    tkgrid(buttonsFrame, columnspan=2, sticky="w")
-    dialogSuffix(rows=8, columns=2)
-    }
-
-scatterPlotMatrixBCA <- function(){
-    require("BCA")
-    initializeDialog(title=gettextRcmdr("Scatterplot Matrix"))
-    variablesBox <- variableListBox(top, Numeric(), title=gettextRcmdr("Select variables (three or more)"),
-        selectmode="multiple", initialSelection=NULL)
-    checkBoxes(frame="optionsFrame", boxes=c("lsLine", "smoothLine", "spread"), initialValues=c(1,1,0),
-        labels=gettextRcmdr(c("Least-squares lines", "Smooth lines", "Show spread")))
-    sliderValue <- tclVar("50")
-    slider <- tkscale(optionsFrame, from=0, to=100, showvalue=TRUE, variable=sliderValue,
-        resolution=5, orient="horizontal")
-    radioButtons(name="diagonal", buttons=c("density", "histogram", "boxplot", "oned", "qqplot", "none"),
-        labels=gettextRcmdr(c("Density plots", "Histograms", "Boxplots", "One-dimensional scatterplots", "Normal QQ plots", "Nothing (empty)")),
-        title=gettextRcmdr("On Diagonal"))
-    subsetBox()
-    onOK <- function(){
-        variables <- getSelection(variablesBox)
-        closeDialog()
-        if (length(variables) < 3) {
-            errorCondition(recall=scatterPlotMatrixBCA, message=gettextRcmdr("Fewer than 3 variable selected."))
-            return()
-            }
-        line <- if("1" == tclvalue(lsLineVariable)) "lm" else "FALSE"
-        smooth <- as.character("1" == tclvalue(smoothLineVariable))
-		spread <- as.character("1" == tclvalue(spreadVariable))
-        span <- as.numeric(tclvalue(sliderValue))
-        diag <- as.character(tclvalue(diagonalVariable))
-        subset <- tclvalue(subsetVariable)
-        subset <- if (trim.blanks(subset) == gettextRcmdr("<all valid cases>")) ""
-            else paste(", subset=", subset, sep="")
-        .activeDataSet <- ActiveDataSet()
-        if (.groups == FALSE) {
-           command <- paste("scatterplotMatrixBCA(~", paste(variables, collapse="+"),
-                ", reg.line=", line, ", smooth=", smooth, ", spread=", spread,
-                ", span=", span/100, ", diagonal = '", diag,
-                "', data=", .activeDataSet, subset, ")", sep="")
-           logger(command)
-           justDoIt(command)
-            }
-        else {
-            command <- paste("scatterplotMatrixBCA(~", paste(variables, collapse="+")," | ", .groups,
-                ", reg.line=", line, ", smooth=", smooth, ", spread=", spread,
-                ", span=", span/100, ", diagonal= '", diag,
-                "', by.groups=", .linesByGroup,
-                ", data=", .activeDataSet, subset, ")", sep="")
-            logger(command)
-            justDoIt(command)
-            }
-        activateMenus()
-        tkfocus(CommanderWindow())
-        }
-    groupsBox(scatterPlot, plotLinesByGroup=TRUE)
-    OKCancelHelp(helpSubject="scatterplotMatrix")
-    tkgrid(getFrame(variablesBox), sticky="nw")
-	tkgrid(labelRcmdr(optionsFrame, text=gettextRcmdr("Span for smooth")), slider, sticky="w")    
-	tkgrid(optionsFrame, sticky="w")
-    tkgrid(diagonalFrame, sticky="w")
-    tkgrid(subsetFrame, sticky="w")
-    tkgrid(groupsFrame, sticky="w")
-    tkgrid(buttonsFrame, columnspan=2, sticky="w")
-    dialogSuffix(rows=6, columns=2)
-    }
-
